unit LibeayFunc;

interface

  uses libeay32, TypesDefs, SysUtils, LogHandler;

type
  PMDValue = ^TMDValue;
  TMDValue = array [0..EVP_MAX_MD_SIZE] of byte;

  procedure BnSetBinary(bn: pBIGNUM; const Arr; len: integer);
  procedure BnModExp(bnR: pBIGNUM; const bn0, bn1, bn2: pBIGNUM);
  procedure BnMul(bnAR: pBIGNUM; const bn1: pBIGNUM);
  procedure BnMod(bnAR: pBIGNUM; const bn1: pBIGNUM);
  procedure BnToBytes(const bn: pBIGNUM; var Arr; len: integer);
  procedure GetSHA1Digest(lpmdValue: PMDValue; var mdLength: dword; parg1: pointer; arg1Len: dword; parg2: pointer; arg2Len: dword);

implementation

uses UnitMain; //log

procedure BnSetBinary(bn: pBIGNUM; const Arr; len: integer);
var
  i,max: integer;
  temp: TBytes;
begin
    Assert(bn <> nil,'BnSetBinary bn nil');
    setlength(temp,len);
    max:= len-1;
    for i:= 0 to max do temp[i]:= TBytes(@Arr)[max - i];
    if BN_bin2bn(temp, len, bn) = nil then
      AddToLog(ERR_error_string(ERR_get_error,nil));
    temp:=nil;
end;

procedure BnModExp(bnR: pBIGNUM; const bn0, bn1, bn2: pBIGNUM);
var
  bnctx: pBN_CTX;
begin
    Assert(bnR <> nil,'BnModExp bnR nil');
    bnctx:= BN_CTX_new();
    if BN_mod_exp(bnR, bn0, bn1, bn2, bnctx) = 0 then
      AddToLog(ERR_error_string(ERR_get_error,nil));
    BN_CTX_free(bnctx);
end;

procedure BnMul(bnAR: pBIGNUM; const bn1: pBIGNUM);
var
  bnctx: pBN_CTX;
begin
    Assert(bnAR <> nil,'BnMul bnAR nil');
    bnctx:= BN_CTX_new();
    if BN_mul(bnAR, bnAR, bn1, bnctx) = 0 then
      AddToLog(ERR_error_string(ERR_get_error,nil));
    BN_CTX_free(bnctx);
end;

procedure BnMod(bnAR: pBIGNUM; const bn1: pBIGNUM);
var
  bnctx: pBN_CTX;
begin
    Assert(bnAR <> nil,'BnMod bnAR nil');
    bnctx:= BN_CTX_new();
    if BN_mod(bnAR, bnAR, bn1, bnctx) = 0 then
      AddToLog(ERR_error_string(ERR_get_error,nil));
    BN_CTX_free(bnctx);
end;

procedure BnToBytes(const bn: pBIGNUM; var Arr; len: integer);
var
  i,max,outlen: integer;
  temp: TBytes;
begin
    Assert(bn <> nil,'BnToBytes bn nil');
    outlen:=BN_num_bytes(bn);
    setlength(temp,outlen);
    if BN_bn2bin(bn,temp) = 0 then
      AddToLog(ERR_error_string(ERR_get_error,nil));
    if len < outlen then
      max:=len-1
    else max:= outlen-1;
    for i:= 0 to max do TByteArray(Arr)[i]:= temp[max - i];
    temp:=nil;
end;

procedure GetSHA1Digest(lpmdValue: PMDValue; var mdLength: dword; parg1: pointer; arg1Len: dword; parg2: pointer; arg2Len: dword);
var
  mdctx: EVP_MD_CTX;
begin
    EVP_DigestInit(@mdctx, EVP_sha1());
    if parg1 <> nil then EVP_DigestUpdate(@mdctx,parg1,arg1Len);
    if parg2 <> nil then EVP_DigestUpdate(@mdctx,parg2,arg2Len);
    EVP_DigestFinal(@mdctx, PAnsiChar(lpmdValue), mdLength);
end;

end.
